<?php
/*
Plugin Name: Custom post types
Plugin URI: https://www.andreadegiovine.it/download/custom-post-types/?utm_source=wordpress_org&utm_medium=plugin_link&utm_campaign=custom_post_types
Description: Create / manage custom post types, custom taxonomies, custom fields and custom templates easily, directly from the WordPress dashboard without writing code.
Author: Andrea De Giovine
Author URI: https://www.andreadegiovine.it/?utm_source=wordpress_org&utm_medium=plugin_details&utm_campaign=custom_post_types
Text Domain: custom-post-types
Domain Path: /languages/
Version: 1.1.2
*/


if ( ! defined( 'ABSPATH' ) ) {
    die( 'Invalid request.' );
}

if ( ! class_exists( 'adg_custom_post_types' ) ) {
    class adg_custom_post_types {
        public $cpt_ui_name = 'manage_cpt';
        public $priority = PHP_INT_MAX;

        public function __construct(){
            register_activation_hook( __FILE__, array( $this, 'plugin_activate' ) );

            // Init main plugin code
            add_action( 'init', array( $this, 'init_load_textdomain' ), -1 );
            add_action( 'init', array( $this, 'init_created_cpt' ), 0 );
            add_action( 'init', array( $this, 'init_created_tax' ), 0 );
            add_filter( 'template_include', array( $this, 'init_created_template' ), 0 );
            add_action( 'init', array( $this, 'init_cpt_ui' ), 10 );
            add_action( 'init', array( $this, 'init_tax_ui' ), 10 );
            add_action( 'init', array( $this, 'init_field_ui' ), 10 );
            add_action( 'init', array( $this, 'init_template_ui' ), 10 );

            // Init columns in dashboard list
            add_filter( 'manage_posts_columns', array( $this, 'init_cpt_admin_columns' ), $this->priority );
            add_action( 'manage_posts_custom_column' , array( $this, 'init_render_cpt_admin_columns' ), $this->priority, 2 );

            // Plugin admin menu
            add_action( 'admin_menu', array( $this, 'init_fix_admin_menu' ), $this->priority );
            add_action( 'admin_enqueue_scripts', array( $this, 'init_admin_cpt_ui_assets' ), $this->priority );

            // UI metaboxes
            add_action( 'add_meta_boxes', array( $this, 'init_cpt_ui_metabox' ), $this->priority );
            add_action( 'save_post', array( $this, 'init_cpt_ui_save_metabox' ), $this->priority );

            // Edit cpt manager page
            add_action( 'edit_form_top', array( $this, 'init_admin_form_top' ), $this->priority );
            add_action( 'edit_form_after_title', array( $this, 'init_admin_form_sub' ), $this->priority );

            // Other
            add_filter( 'wp_insert_post_data' , array( $this, 'init_cpt_ui_unique_name' ), $this->priority, 2 );
            add_action( 'admin_notices', array( $this, 'init_admin_notices' ), $this->priority);
            add_filter( 'plugin_action_links', array( $this, 'init_admin_action_links' ), $this->priority, 2);

            // Init shortcode
            add_shortcode( 'custom-field', array( $this, 'field_shortcode' ) );
        }

        public function init_load_textdomain() {
            load_plugin_textdomain( 'custom-post-types', false, dirname( plugin_basename( __FILE__ ) ) . '/languages' ); 
        }

        public function plugin_activate() {
            $upload = wp_upload_dir();
            $upload_dir = $upload['basedir'];
            $upload_dir = $upload_dir . '/custom-templates';
            if (! is_dir($upload_dir)) {
                mkdir( $upload_dir, 0700 );
            }
        }

        public function init_cpt_ui(){
            $labels = array(
                'name'               => _x( 'Custom post types', 'Dashboard menu', 'custom-post-types' ),
                'singular_name'      => __( 'Post type', 'custom-post-types' ),
                'menu_name'          => __( 'Post types', 'custom-post-types' ),
                'name_admin_bar'     => __( 'Post type', 'custom-post-types' ),
                'add_new'            => __( 'Add post type', 'custom-post-types' ),
                'add_new_item'       => __( 'Add new post type', 'custom-post-types' ),
                'new_item'           => __( 'New post type', 'custom-post-types' ),
                'edit_item'          => __( 'Edit post type', 'custom-post-types' ),
                'view_item'          => __( 'View post type', 'custom-post-types' ),
                'all_items'          => _x( 'Custom post types', 'Dashboard menu', 'custom-post-types' ),
                'search_items'       => __( 'Search post type', 'custom-post-types' ),
                'not_found'          => __( 'No post type available.', 'custom-post-types' ),
                'not_found_in_trash' => __( 'No post type in the trash.', 'custom-post-types' )
            );

            $args = array(
                'labels'             => $labels,
                'description'        => __( 'Create and manage custom post types.', 'custom-post-types' ),
                'public'             => false,
                'publicly_queryable' => false,
                'show_ui'            => true,
                'show_in_menu'       => true,
                'query_var'          => false,
                'rewrite'            => false,
                'capabilities' => array(
                    'edit_post'          => 'update_core',
                    'read_post'          => 'update_core',
                    'delete_post'        => 'update_core',
                    'edit_posts'         => 'update_core',
                    'edit_others_posts'  => 'update_core',
                    'delete_posts'       => 'update_core',
                    'publish_posts'      => 'update_core',
                    'read_private_posts' => 'update_core'
                ),
                'has_archive'        => false,
                'hierarchical'       => false,
                'menu_position'      => null,
                'supports'           => array( 'title' ),
                'menu_icon'          => 'dashicons-index-card'
            );
            register_post_type( $this->cpt_ui_name, $args );
        }

        public function init_tax_ui(){
            $labels = array(
                'name'               => __( 'Custom taxonomies', 'custom-post-types' ),
                'singular_name'      => __( 'Taxonomy', 'custom-post-types' ),
                'menu_name'          => __( 'Taxonomy', 'custom-post-types' ),
                'name_admin_bar'     => __( 'Taxonomy', 'custom-post-types' ),
                'add_new'            => __( 'Add taxonomy', 'custom-post-types' ),
                'add_new_item'       => __( 'Add new taxonomy', 'custom-post-types' ),
                'new_item'           => __( 'New taxonomy', 'custom-post-types' ),
                'edit_item'          => __( 'Edit taxonomy', 'custom-post-types' ),
                'view_item'          => __( 'View taxonomy', 'custom-post-types' ),
                'all_items'          => __( 'Custom taxonomies', 'custom-post-types' ),
                'search_items'       => __( 'Search taxonomy', 'custom-post-types' ),
                'not_found'          => __( 'No taxonomy available.', 'custom-post-types' ),
                'not_found_in_trash' => __( 'No taxonomy in the trash.', 'custom-post-types' )
            );

            $args = array(
                'labels'             => $labels,
                'description'        => __( 'Create and manage custom taxonomies.', 'custom-post-types' ),
                'public'             => false,
                'publicly_queryable' => false,
                'show_ui'            => true,
                'show_in_menu'       => true,
                'query_var'          => false,
                'rewrite'            => false,
                'capabilities' => array(
                    'edit_post'          => 'update_core',
                    'read_post'          => 'update_core',
                    'delete_post'        => 'update_core',
                    'edit_posts'         => 'update_core',
                    'edit_others_posts'  => 'update_core',
                    'delete_posts'       => 'update_core',
                    'publish_posts'      => 'update_core',
                    'read_private_posts' => 'update_core'
                ),
                'has_archive'        => false,
                'hierarchical'       => false,
                'menu_position'      => null,
                'supports'           => array( 'title' ),
                'menu_icon'          => 'dashicons-index-card',
                'show_in_menu'       => 'edit.php?post_type=' . $this->cpt_ui_name
            );
            register_post_type( $this->cpt_ui_name . '_tax', $args );
        }

        public function init_field_ui(){
            $labels = array(
                'name'               => __( 'Custom fields', 'custom-post-types' ),
                'singular_name'      => __( 'Field', 'custom-post-types' ),
                'menu_name'          => __( 'Field', 'custom-post-types' ),
                'name_admin_bar'     => __( 'Field', 'custom-post-types' ),
                'add_new'            => __( 'Add field', 'custom-post-types' ),
                'add_new_item'       => __( 'Add new field', 'custom-post-types' ),
                'new_item'           => __( 'New field', 'custom-post-types' ),
                'edit_item'          => __( 'Edit field', 'custom-post-types' ),
                'view_item'          => __( 'View field', 'custom-post-types' ),
                'all_items'          => __( 'Custom fields', 'custom-post-types' ),
                'search_items'       => __( 'Search Field', 'custom-post-types' ),
                'not_found'          => __( 'No field available.', 'custom-post-types' ),
                'not_found_in_trash' => __( 'No field in the trash.', 'custom-post-types' )
            );

            $args = array(
                'labels'             => $labels,
                'description'        => __( 'Create and manage custom fields.', 'custom-post-types' ),
                'public'             => false,
                'publicly_queryable' => false,
                'show_ui'            => true,
                'show_in_menu'       => true,
                'query_var'          => false,
                'rewrite'            => false,
                'capabilities' => array(
                    'edit_post'          => 'update_core',
                    'read_post'          => 'update_core',
                    'delete_post'        => 'update_core',
                    'edit_posts'         => 'update_core',
                    'edit_others_posts'  => 'update_core',
                    'delete_posts'       => 'update_core',
                    'publish_posts'      => 'update_core',
                    'read_private_posts' => 'update_core'
                ),
                'has_archive'        => false,
                'hierarchical'       => false,
                'menu_position'      => null,
                'supports'           => array( 'title' ),
                'menu_icon'          => 'dashicons-index-card',
                'show_in_menu'       => 'edit.php?post_type=' . $this->cpt_ui_name
            );
            register_post_type( $this->cpt_ui_name . '_field', $args );
        }

        public function init_template_ui(){
            $labels = array(
                'name'               => __( 'Custom templates', 'custom-post-types' ),
                'singular_name'      => __( 'Template', 'custom-post-types' ),
                'menu_name'          => __( 'Template', 'custom-post-types' ),
                'name_admin_bar'     => __( 'Template', 'custom-post-types' ),
                'add_new'            => __( 'Add template', 'custom-post-types' ),
                'add_new_item'       => __( 'Add new template', 'custom-post-types' ),
                'new_item'           => __( 'New template', 'custom-post-types' ),
                'edit_item'          => __( 'Edit template', 'custom-post-types' ),
                'view_item'          => __( 'View template', 'custom-post-types' ),
                'all_items'          => __( 'Custom template', 'custom-post-types' ),
                'search_items'       => __( 'Search template', 'custom-post-types' ),
                'not_found'          => __( 'No template available.', 'custom-post-types' ),
                'not_found_in_trash' => __( 'No template in the trash.', 'custom-post-types' )
            );

            $args = array(
                'labels'             => $labels,
                'description'        => __( 'Create and manage custom templates.', 'custom-post-types' ),
                'public'             => true,
                'publicly_queryable' => false,
                'show_ui'            => true,
                'show_in_menu'       => true,
                'query_var'          => false,
                'rewrite'            => false,
                'capabilities' => array(
                    'edit_post'          => 'update_core',
                    'read_post'          => 'update_core',
                    'delete_post'        => 'update_core',
                    'edit_posts'         => 'update_core',
                    'edit_others_posts'  => 'update_core',
                    'delete_posts'       => 'update_core',
                    'publish_posts'      => 'update_core',
                    'read_private_posts' => 'update_core'
                ),
                'has_archive'        => false,
                'hierarchical'       => false,
                'menu_position'      => null,
                'supports'           => array( 'title', 'editor' ),
                'menu_icon'          => 'dashicons-index-card',
                'show_in_menu'       => 'edit.php?post_type=' . $this->cpt_ui_name
            );
            register_post_type( $this->cpt_ui_name . '_template', $args );



            $current_theme_name = wp_get_theme()->get( 'TextDomain' );
            $upload = wp_upload_dir();
            $upload_dir = $upload['basedir'];
            $current_teme_single_template = $upload_dir . '/custom-templates/' . $current_theme_name .'-single.php';
            if (!file_exists($current_teme_single_template) && (isset($_GET['action']) && 'copy_theme_template' == $_GET['action'] )) {

                $theme_single_template = file_get_contents(get_template_directory() . '/single.php');
                /*$theme_single_template = preg_replace("/[^<?php while ( have_posts() ) : ?>](.*)[^<?php endwhile; ?>]/",'Test',$theme_single_template);*/

                if(preg_match('#while(.+)endwhile#s',$theme_single_template,$matches)){
                    $new_single_template = preg_replace( '#while(.+)endwhile#s', "custom_post_types_get_custom_template()", $theme_single_template);

                    $new_file = fopen($current_teme_single_template, "w") or die("Unable to create the template file! Check permission.");
                    fwrite($new_file, $new_single_template);
                    fclose($new_file);

                    wp_redirect( admin_url( 'edit.php?post_type='.$this->cpt_ui_name.'_template' ) );
                    exit;

                }

            }
        }

        public function init_fix_admin_menu(){
            remove_submenu_page('edit.php?post_type=' . $this->cpt_ui_name,'post-new.php?post_type=' . $this->cpt_ui_name);
            //add_submenu_page( 'edit.php?post_type=' . $this->cpt_ui_name, __( 'Custom fields', 'custom-post-types' ),  __( 'Custom Fields', 'custom-post-types' ), 'manage_options', 'go-custom-fields', array($this, 'work_in_progress'));
            //add_submenu_page( 'edit.php?post_type=' . $this->cpt_ui_name, __( 'Custom templates', 'custom-post-types' ),  __( 'Custom templates', 'custom-post-types' ), 'manage_options', 'go-custom-templates', array($this, 'work_in_progress'));
        }

        public function init_admin_cpt_ui_assets() {
            wp_enqueue_style( $this->cpt_ui_name . '_css', plugins_url('assets/style.css', __FILE__) );
            wp_enqueue_script( $this->cpt_ui_name . '_js', plugins_url('assets/script.js', __FILE__), array( 'jquery' ) );
            //add_thickbox();
            //wp_enqueue_media();
        }

        public function init_cpt_ui_metabox( $post_type ) {
            if ( $post_type == $this->cpt_ui_name ) {
                add_meta_box(
                    'cpt_ui_metabox',
                    __( 'Post type settings', 'custom-post-types' ),
                    array( $this, 'init_cpt_ui_render_metabox' ),
                    $post_type,
                    'normal',
                    'high'
                );
            } elseif ( $post_type == $this->cpt_ui_name . '_tax' ) {
                add_meta_box(
                    'tax_ui_metabox',
                    __( 'Taxonomy settings', 'custom-post-types' ),
                    array( $this, 'init_tax_ui_render_metabox' ),
                    $post_type,
                    'normal',
                    'high'
                );
            } elseif ( $post_type == $this->cpt_ui_name . '_field' ) {
                add_meta_box(
                    'field_ui_metabox',
                    __( 'Field settings', 'custom-post-types' ),
                    array( $this, 'init_field_ui_render_metabox' ),
                    $post_type,
                    'normal',
                    'high'
                );
            } elseif ( $post_type == $this->cpt_ui_name . '_template' ) {
                add_meta_box(
                    'template_ui_metabox',
                    __( 'Template settings', 'custom-post-types' ),
                    array( $this, 'init_template_ui_render_metabox' ),
                    $post_type,
                    'side',
                    'low'
                );
            }





            // Search for custom field box for current post type

            $all_custom_metaboxes = get_posts( array( 'posts_per_page' => -1, 'post_type' => $this->cpt_ui_name . '_field' ) );
            foreach($all_custom_metaboxes as $metaboxes){

                $field_settings = get_post_meta( $metaboxes->ID, '_field_settings_meta')[0];

                if(isset($field_settings['field_post_types']) && in_array($post_type, $field_settings['field_post_types'])){

                    $box_id = $metaboxes->ID;
                    $post_type_name = get_post_type_object($post_type)->labels->singular_name;

                    add_meta_box(
                        $box_id . '_metabox',
                        $metaboxes->post_title,
                        array( $this, 'render_custom_metabox' ),
                        $post_type,
                        isset($field_settings['field_position']) && $field_settings['field_position'] == 'side' ? 'side' : 'normal',
                        'high'
                    );

                }
            }







        }

        public function init_cpt_ui_render_metabox( $post ) {
            wp_nonce_field( 'cpt_ui_inner_metabox', 'cpt_ui_metabox_nonce' );
            $cpt_settings = isset(get_post_meta( $post->ID, '_cpt_settings_meta')[0]) ? get_post_meta( $post->ID, '_cpt_settings_meta')[0] : array();

            $cpt_singular_name = isset($cpt_settings['cpt_singular_name']) ? $cpt_settings['cpt_singular_name'] : '';
            $cpt_hierarchical = isset($cpt_settings['cpt_hierarchical']) && $cpt_settings['cpt_hierarchical'] == 0 ? 0 : 1;
            $cpt_public = isset($cpt_settings['cpt_public']) && $cpt_settings['cpt_public'] == 0 ? 0 : 1;
            $cpt_editor = isset($cpt_settings['cpt_editor']) && $cpt_settings['cpt_editor'] == 0 ? 0 : 1;
            $cpt_excerpt = isset($cpt_settings['cpt_excerpt']) && $cpt_settings['cpt_excerpt'] == 0 ? 0 : 1;
            $cpt_thumbnail = isset($cpt_settings['cpt_thumbnail']) && $cpt_settings['cpt_thumbnail'] == 0 ? 0 : 1;
            $cpt_icon = isset($cpt_settings['cpt_icon']) ? $cpt_settings['cpt_icon'] : '';
            $cpt_role = isset($cpt_settings['cpt_role']) && $cpt_settings['cpt_role'] == 0 ? 0 : 1;
?>
<div class="wp-cpt-row">
    <label for="cpt_singular_name">
        <?php _e( 'Single name', 'custom-post-types' ); ?>
    </label>
    <input type="text" id="cpt_singular_name" name="cpt_settings[cpt_singular_name]" value="<?php echo esc_attr( $cpt_singular_name ); ?>" placeholder="<?php _e( 'ex: Product', 'custom-post-types' ); ?>" required />
    <small><?php _e( 'Singular name.', 'custom-post-types' ); ?></small>
</div>

<div class="wp-cpt-row">
    <label for="cpt_hierarchical">
        <?php _e( 'Hierarchical', 'custom-post-types' ); ?>
    </label>
    <select id="cpt_hierarchical" name="cpt_settings[cpt_hierarchical]">
        <option value="1" <?php selected( $cpt_hierarchical, 1 ); ?>><?php _e( 'YES', 'custom-post-types' ); ?></option>
        <option value="0" <?php selected( $cpt_hierarchical, 0 ); ?>><?php _e( 'NO', 'custom-post-types' ); ?></option>
    </select>    
    <small><?php _e( 'If set to "YES" it will be possible to set a parent POST TYPE (as for pages).', 'custom-post-types' ); ?></small>
</div>

<div class="wp-cpt-row">
    <label for="cpt_public">
        <?php _e( 'Visibility', 'custom-post-types' ); ?>
    </label>
    <select id="cpt_public" name="cpt_settings[cpt_public]">
        <option value="1" <?php selected( $cpt_public, 1 ); ?>><?php _e( 'YES', 'custom-post-types' ); ?></option>
        <option value="0" <?php selected( $cpt_public, 0 ); ?>><?php _e( 'NO', 'custom-post-types' ); ?></option>
    </select>    
    <small><?php _e( 'If set to "YES" it will be shown in the frontend and will have a permalink and a single template.', 'custom-post-types' ); ?></small>
</div>

<div class="wp-cpt-row">
    <label for="cpt_editor">
        <?php _e( 'Content editor', 'custom-post-types' ); ?>
    </label>
    <select id="cpt_editor" name="cpt_settings[cpt_editor]">
        <option value="1" <?php selected( $cpt_editor, 1 ); ?>><?php _e( 'YES', 'custom-post-types' ); ?></option>
        <option value="0" <?php selected( $cpt_editor, 0 ); ?>><?php _e( 'NO', 'custom-post-types' ); ?></option>
    </select>    
    <small><?php _e( 'If set to "YES" the content editor will be shown during creation / modification.', 'custom-post-types' ); ?></small>
</div>

<div class="wp-cpt-row">
    <label for="cpt_excerpt">
        <?php _e( 'Excerpt', 'custom-post-types' ); ?>
    </label>
    <select id="cpt_excerpt" name="cpt_settings[cpt_excerpt]">
        <option value="1" <?php selected( $cpt_excerpt, 1 ); ?>><?php _e( 'YES', 'custom-post-types' ); ?></option>
        <option value="0" <?php selected( $cpt_excerpt, 0 ); ?>><?php _e( 'NO', 'custom-post-types' ); ?></option>
    </select>    
    <small><?php _e( 'If set to "YES" the "Excerpt" field will be shown during creation / modification.', 'custom-post-types' ); ?></small>
</div>

<div class="wp-cpt-row">
    <label for="cpt_thumbnail">
        <?php _e( 'Featured image', 'custom-post-types' ); ?>
    </label>
    <select id="cpt_thumbnail" name="cpt_settings[cpt_thumbnail]">
        <option value="1" <?php selected( $cpt_thumbnail, 1 ); ?>><?php _e( 'YES', 'custom-post-types' ); ?></option>
        <option value="0" <?php selected( $cpt_thumbnail, 0 ); ?>><?php _e( 'NO', 'custom-post-types' ); ?></option>
    </select>    
    <small><?php _e( 'If set to "YES" the "Featured image" field will be shown during creation / modification.', 'custom-post-types' ); ?></small>
</div>

<div class="wp-cpt-row">
    <label for="cpt_icon">
        <?php _e( 'Menu icon', 'custom-post-types' ); ?>
    </label>
    <input type="text" id="cpt_icon" name="cpt_settings[cpt_icon]" value="<?php echo esc_attr( $cpt_icon ); ?>" placeholder="dashicons-tag" />
    <small><?php _e( 'You can enter the name of an <a href="https://developer.wordpress.org/resource/dashicons" target="_blank" rel="nofolow">Dashicons</a> icon (ex: dashicons-tag) or the URL of an image (20px * 20px).', 'custom-post-types' ); ?></small>
</div>

<div class="wp-cpt-row">
    <label for="cpt_role">
        <?php _e( 'Administrators only', 'custom-post-types' ); ?>
    </label>
    <select id="cpt_role" name="cpt_settings[cpt_role]">
        <option value="1" <?php selected( $cpt_role, 1 ); ?>><?php _e( 'YES', 'custom-post-types' ); ?></option>
        <option value="0" <?php selected( $cpt_role, 0 ); ?>><?php _e( 'NO', 'custom-post-types' ); ?></option>
    </select>    
    <small><?php _e( 'If set to "YES", only the administrators can create / modify these contents, if "NO" all the roles with the minimum capacity of "edit_posts".', 'custom-post-types' ); ?></small>
</div>
<?php
        }

        public function init_tax_ui_render_metabox( $post ) {
            wp_nonce_field( 'tax_ui_inner_metabox', 'tax_ui_metabox_nonce' );
            $tax_settings = isset(get_post_meta( $post->ID, '_tax_settings_meta')[0]) ? get_post_meta( $post->ID, '_tax_settings_meta')[0] : array();

            $tax_singular_name = isset($tax_settings['tax_singular_name']) ? $tax_settings['tax_singular_name'] : '';
            $tax_hierarchical = isset($tax_settings['tax_hierarchical']) && $tax_settings['tax_hierarchical'] == 0 ? 0 : 1;
            $tax_public = isset($tax_settings['tax_public']) && $tax_settings['tax_public'] == 0 ? 0 : 1;
            $tax_post_types = isset($tax_settings['tax_post_types']) ? $tax_settings['tax_post_types'] : array();
            $tax_role = isset($tax_settings['tax_role']) && $tax_settings['tax_role'] == 0 ? 0 : 1;
?>
<div class="wp-cpt-row">
    <label for="tax_singular_name">
        <?php _e( 'Single name', 'custom-post-types' ); ?>
    </label>
    <input type="text" id="tax_singular_name" name="tax_settings[tax_singular_name]" value="<?php echo esc_attr( $tax_singular_name ); ?>" placeholder="<?php _e( 'ex: Product Category', 'custom-post-types' ); ?>" required />
    <small><?php _e( 'Singular name.', 'custom-post-types' ); ?></small>
</div>

<div class="wp-cpt-row">
    <label for="tax_hierarchical">
        <?php _e( 'Hierarchical', 'custom-post-types' ); ?>
    </label>
    <select id="tax_hierarchical" name="tax_settings[tax_hierarchical]">
        <option value="1" <?php selected( $tax_hierarchical, 1 ); ?>><?php _e( 'YES', 'custom-post-types' ); ?></option>
        <option value="0" <?php selected( $tax_hierarchical, 0 ); ?>><?php _e( 'NO', 'custom-post-types' ); ?></option>
    </select>    
    <small><?php _e( 'If set to "YES" it will be possible to set a parent TAXONOMY (as for the posts categories).', 'custom-post-types' ); ?></small>
</div>

<div class="wp-cpt-row">
    <label for="tax_public">
        <?php _e( 'Visibility', 'custom-post-types' ); ?>
    </label>
    <select id="tax_public" name="tax_settings[tax_public]">
        <option value="1" <?php selected( $tax_public, 1 ); ?>><?php _e( 'YES', 'custom-post-types' ); ?></option>
        <option value="0" <?php selected( $tax_public, 0 ); ?>><?php _e( 'NO', 'custom-post-types' ); ?></option>
    </select>    
    <small><?php _e( 'If set to "YES" it will be shown in the frontend and will have a permalink and an archive template.', 'custom-post-types' ); ?></small>
</div>

<div class="wp-cpt-row">
    <label for="tax_post_types">
        <?php _e( 'Assignment', 'custom-post-types' ); ?>
    </label>
    <div>
        <div><input type="checkbox" name="tax_settings[tax_post_types][]" value="post" <?php echo ( in_array("post", $tax_post_types) ? 'checked' : '');?>> <?php _e( 'Posts', 'custom-post-types' ); ?></div>
        <?php
            $post_types = get_post_types( array('_builtin' => false), 'objects');
            foreach ( $post_types  as $post_type ) { if($post_type->name != $this->cpt_ui_name && $post_type->name != $this->cpt_ui_name . '_tax' && $post_type->name != $this->cpt_ui_name . '_field') { ?>
        <div><input type="checkbox" name="tax_settings[tax_post_types][]" value="<?php echo $post_type->name; ?>" <?php echo ( in_array($post_type->name, $tax_post_types) ? 'checked' : '');?>> <?php echo $post_type->label; ?></div>
        <?php } }
        ?>
    </div>
    <small><?php _e( 'Choose for which POST TYPE use this taxonomy.', 'custom-post-types' ); ?></small>
</div>

<div class="wp-cpt-row">
    <label for="tax_role">
        <?php _e( 'Administrators only', 'custom-post-types' ); ?>
    </label>
    <select id="tax_role" name="tax_settings[tax_role]">
        <option value="1" <?php selected( $tax_role, 1 ); ?>><?php _e( 'YES', 'custom-post-types' ); ?></option>
        <option value="0" <?php selected( $tax_role, 0 ); ?>><?php _e( 'NO', 'custom-post-types' ); ?></option>
    </select>    
    <small><?php _e( 'If set to "YES", only the administrators can create / modify these contents, if "NO" all the roles with the minimum capacity of "edit_posts".', 'custom-post-types' ); ?></small>
</div>
<?php
        }

        public function init_field_ui_render_metabox( $post ) {
            wp_nonce_field( 'field_ui_inner_metabox', 'field_ui_metabox_nonce' );
            $field_settings = isset(get_post_meta( $post->ID, '_field_settings_meta')[0]) ? get_post_meta( $post->ID, '_field_settings_meta')[0] : array();

            $field_position = isset($field_settings['field_position']) && $field_settings['field_position'] == 'side' ? 'side' : 'normal';
            $field_post_types = isset($field_settings['field_post_types']) ? $field_settings['field_post_types'] : array();
            $field_role = isset($field_settings['field_role']) && $field_settings['field_role'] == 0 ? 0 : 1;
            $fields = isset($field_settings['fields']) ? $field_settings['fields'] : array();
?>

<div class="wp-cpt-row">
    <label for="field_position">
        <?php _e( 'Position', 'custom-post-types' ); ?>
    </label>
    <select id="field_position" name="field_settings[field_position]">
        <option value="normal" <?php selected( $field_position, 'normal' ); ?>><?php _e( 'EDITOR', 'custom-post-types' ); ?></option>
        <option value="side" <?php selected( $field_position, 'side' ); ?>><?php _e( 'SIDEBAR', 'custom-post-types' ); ?></option>
    </select>    
    <small><?php _e( 'If set to "EDITOR" it will be shown at the bottom of the central column, if "SIDEBAR" it will be shown in the sidebar.', 'custom-post-types' ); ?></small>
</div>

<div class="wp-cpt-row">
    <label for="field_post_types">
        <?php _e( 'Assignment', 'custom-post-types' ); ?>
    </label>
    <div>
        <div><input type="checkbox" name="field_settings[field_post_types][]" value="post" <?php echo ( in_array("post", $field_post_types) ? 'checked' : '');?>> <?php _e( 'Posts', 'custom-post-types' ); ?></div>
        <?php
            $post_types = get_post_types( array('_builtin' => false), 'objects');
            foreach ( $post_types  as $post_type ) { if($post_type->name != $this->cpt_ui_name && $post_type->name != $this->cpt_ui_name . '_field'  && $post_type->name != $this->cpt_ui_name . '_tax') { ?>
        <div><input type="checkbox" name="field_settings[field_post_types][]" value="<?php echo $post_type->name; ?>" <?php echo ( in_array($post_type->name, $field_post_types) ? 'checked' : '');?>> <?php echo $post_type->label; ?></div>
        <?php } }
        ?>
    </div>
    <small><?php _e( 'Choose for which POST TYPE use this field.', 'custom-post-types' ); ?></small>
</div>

<div class="wp-cpt-row">
    <label for="field_role">
        <?php _e( 'Administrators only', 'custom-post-types' ); ?>
    </label>
    <select id="field_role" name="field_settings[field_role]">
        <option value="1" <?php selected( $field_role, 1 ); ?>><?php _e( 'YES', 'custom-post-types' ); ?></option>
        <option value="0" <?php selected( $field_role, 0 ); ?>><?php _e( 'NO', 'custom-post-types' ); ?></option>
    </select>    
    <small><?php _e( 'If set to "YES", only the administrators can create / modify these contents, if "NO" all the roles with the minimum capacity of "edit_posts".', 'custom-post-types' ); ?></small>
</div>

<h3><?php _e( 'Fields list', 'custom-post-types' ); ?></h3>
<p><?php _e( 'Add the custom fields you want to use when creating / editing the associated content types.<br>
To display the field in your template use the shortcode [custom-field id="field_XXXX"] that appears next to the created field.', 'custom-post-types' ); ?><br><?php _e( 'You can use the function included <strong>get_custom_field("field_XXXX")</strong> which returns the value of the field for the current post, or alternatively you can indicate a post ID to extract the value of the field <strong>get_custom_field("field_XXXX ", 66)</strong>.', 'custom-post-types' ); ?></p>

<?php foreach($fields as $id => $field){ if($field['type'] !== '0'){ ?>
<div class="wp-cpt-row field-form" id="<?php echo $id;?>">
    <div>
        <select class="field-type" name="field_settings[fields][<?php echo $id;?>][type]">
            <option value="0"><?php _e( 'Field type...', 'custom-post-types' ); ?></option>
            <option value="text" <?php selected( $field['type'], 'text' ); ?>><?php _e( 'TEXT', 'custom-post-types' ); ?></option>
            <option value="number" <?php selected( $field['type'], 'number' ); ?>><?php _e( 'NUMBER', 'custom-post-types' ); ?></option>
            <option value="textarea" <?php selected( $field['type'], 'textarea' ); ?>><?php _e( 'TEXTAREA', 'custom-post-types' ); ?></option>
            <option value="email" <?php selected( $field['type'], 'email' ); ?>><?php _e( 'EMAIL', 'custom-post-types' ); ?></option>
            <option value="dropdown" <?php selected( $field['type'], 'dropdown' ); ?>><?php _e( 'DROPDOWN', 'custom-post-types' ); ?></option>
            <option value="date" <?php selected( $field['type'], 'date' ); ?>><?php _e( 'DATE', 'custom-post-types' ); ?></option>
            <option value="time" <?php selected( $field['type'], 'time' ); ?>><?php _e( 'TIME', 'custom-post-types' ); ?></option>
            <option value="file" <?php selected( $field['type'], 'file' ); ?>><?php _e( 'FILE UPLOAD', 'custom-post-types' ); ?></option>
        </select>
        <div class="delete-field"><a href="#">x <?php _e( 'delete', 'custom-post-types' ); ?></a></div>
    </div>
    <div><strong>[custom-field id="<?php echo $id;?>"]</strong></div>
    <div class="field-container">
        <input type="text" name="field_settings[fields][<?php echo $id;?>][name]" placeholder="<?php _e( 'Field name', 'custom-post-types' ); ?>" value="<?php echo $field['name'];?>"><br>
        <label><input type="checkbox" name="field_settings[fields][<?php echo $id;?>][required]" value="1" <?php checked( (isset($field['required']) ? $field['required'] : 0), 1 ); ?>> <?php _e( 'Required', 'custom-post-types' ); ?></label>

        <?php if($field['type'] == 'dropdown' ){ ?>
        <br><textarea name="field_settings[fields][<?php echo $id;?>][options]" placeholder="<?php _e( 'Field options, one per row', 'custom-post-types' ); ?>"><?php echo $field['options'];?></textarea>
        <?php } ?>

        <?php /*if($field['type'] == 'file' ){ ?>
        <br><textarea name="field_settings[fields][<?php echo $id;?>][options]" placeholder="<?php _e( 'Type of file, one per row', 'custom-post-types' ); ?>"><?php echo $field['options'];?></textarea>
        <?php }*/ ?>
    </div>
</div>
<?php } else {



        }} ?>

<a href="#" class="button button-primary add-field"><?php _e( 'Add field', 'custom-post-types' ); ?></a>

<script>

    jQuery( document ).ready( function( $ ) {

        var field_form_base = $('.wp-cpt-row.field-form').html();

        $( ".add-field" ).click(function(e) {
            e.preventDefault();
            var random_name = "field_" + Math.random().toString(36).substring(7); // Math.floor((Math.random() * 10000) + 1000);

            var default_field = '<div class="wp-cpt-row field-form" id="' + random_name + '">' +
                '<div>' +
                '<select class="field-type" name="field_settings[fields][' + random_name + '][type]">' +
                '<option value="0"><?php _e( 'Field type...', 'custom-post-types' ); ?></option>' +
                '<option value="text"><?php _e( 'TEXT', 'custom-post-types' ); ?></option>' +
                '<option value="number"><?php _e( 'NUMBER', 'custom-post-types' ); ?></option>' +
                '<option value="textarea"><?php _e( 'TEXTAREA', 'custom-post-types' ); ?></option>' +
                '<option value="email"><?php _e( 'EMAIL', 'custom-post-types' ); ?></option>' +
                '<option value="dropdown"><?php _e( 'DROPDOWN', 'custom-post-types' ); ?></option>' +
                '<option value="date"><?php _e( 'DATE', 'custom-post-types' ); ?></option>' +
                '<option value="time"><?php _e( 'TIME', 'custom-post-types' ); ?></option>' +
                '<option value="file"><?php _e( 'FILE UPLOAD', 'custom-post-types' ); ?></option>' +
                '</select> ' +
                '</div>' +
                '<div><strong>[custom-field id="' + random_name + '"]</strong></div>' +
                '<div class="field-container"><?php _e( 'Select the field type first.', 'custom-post-types' ); ?></div> ' +
                '</div>';

            $(default_field).insertBefore( this );
        });


        $('.inside').on('change', '.field-form .field-type', function(){
            var input_type = $(this).val();
            var field_form = $( this ).closest('.field-form');
            var field_container = $( field_form ).find('.field-container');
            var field_id = $( field_form ).attr('id');
            var field_settings = '<?php _e( 'Select the field type first.', 'custom-post-types' ); ?>';
            var remove_field = '<div class="delete-field"><a href="#">x <?php _e( 'delete', 'custom-post-types' ); ?></a></div>';

            if(input_type != 0 ){
                field_settings = '<input type="text" name="field_settings[fields][' + field_id + '][name]" placeholder="<?php _e( 'Field name', 'custom-post-types' ); ?>"><br>' +
                    '<label><input type="checkbox" name="field_settings[fields][' + field_id + '][required]" value="1"> <?php _e( 'Required', 'custom-post-types' ); ?></label>';
                if($(this).closest('div').find('a').length == 0){
                    $(remove_field).insertAfter( $(this) );
                }
            } else {
                $(this).closest('div').find('a').remove();
            }

            if(input_type == 'dropdown'){
                field_settings += '<br><textarea name="field_settings[fields][' + field_id + '][options]" placeholder="<?php _e( 'Field options, one per row', 'custom-post-types' ); ?>"></textarea>';
            }

            $(field_container).html(field_settings);
        });

        $('.inside').on('click', '.delete-field a', function(e){
            e.preventDefault();
            var field_form = $( this ).closest('.field-form');
            var field_type = $( field_form ).find('.field-type');
            var field_container = $( field_form ).find('.field-container');
            var field_settings = '<?php _e( 'Select the field type first.', 'custom-post-types' ); ?>';
            $(field_type).val(0);
            $(field_container).html(field_settings);
            $(this).remove();
        });

    } );

</script>
<?php
        }



        public function init_template_ui_render_metabox( $post ) {
            wp_nonce_field( 'template_ui_inner_metabox', 'template_ui_metabox_nonce' );
            $template_settings = isset(get_post_meta( $post->ID, '_template_settings_meta')[0]) ? get_post_meta( $post->ID, '_template_settings_meta')[0] : array();

            $template_used_by = isset($template_settings['field_used_by']) ? $template_settings['field_used_by'] : '';
?>

<div class="wp-field-row">
    <label for="used_by" title="<?php _e( 'Used by', 'custom-post-types' ); ?>"><?php _e( 'Used by', 'custom-post-types' ); ?></label>
    <select id="used_by" name="template_settings[field_used_by]">
        <option value=""><?php _e( '- Choose -', 'custom-post-types' ); ?></option>
        <optgroup label="<?php _e( 'Post types', 'custom-post-types' ); ?>">
            <?php
            $ignore_post_type = array();
            $all_custom_templates = get_posts( array( 'posts_per_page' => -1, 'post_type' => $this->cpt_ui_name . '_template' ) );
            foreach($all_custom_templates as $template){
                $template_settings = get_post_meta( $template->ID, '_template_settings_meta')[0];
                if(!empty($template_settings['field_used_by']) && $template->ID != get_the_ID()){
                    $ignore_post_type[] = $template_settings['field_used_by'];
                }
            }


            $created_post_types = get_posts( array('posts_per_page' => -1, 'post_type' => $this->cpt_ui_name) );

            foreach($created_post_types as $post_type){
                if(!in_array("cpt_".$post_type->ID, $ignore_post_type)){
                    echo '<option value="cpt_'.$post_type->ID.'" '.selected( $template_used_by, 'cpt_'.$post_type->ID ).'>'.$post_type->post_title.'</option>';
                }
            } 
            ?>
        </optgroup>
    </select>
</div>

<?php if(!empty($template_used_by)){ 


                $all_custom_metaboxes = get_posts( array( 'posts_per_page' => -1, 'post_type' => $this->cpt_ui_name . '_field' ) );
                foreach($all_custom_metaboxes as $metaboxes){

                    $field_settings = get_post_meta( $metaboxes->ID, '_field_settings_meta')[0];
                    $fields = isset($field_settings['fields']) ? $field_settings['fields'] : array();
                    if(isset($field_settings['field_post_types']) && in_array($template_used_by, $field_settings['field_post_types'])){
                        echo "<hr><p>".__( 'Add the custom fields you want to use when creating / editing the associated content types.<br>
To display the field in your template use the shortcode [custom-field id="field_XXXX"] that appears next to the created field.', 'custom-post-types' )."</p>";
                        echo "<p style=\"text-align: center; text-transform: uppercase;\"><strong>".__( 'Default field', 'custom-post-types' )."</strong></p>";
                        echo '<p>
    <label for="custom_field_title" title="'.__( 'Post title', 'custom-post-types' ).'"><strong>'.__( 'Post title', 'custom-post-types' ).'</strong></label><br>
    <input type="text" class="shortcode-custom-fields" value=\'[custom-field id="title"]\' readonly="readonly" onfocus="this.select();" id="custom_field_title">
</p>';
                        echo '<p>
    <label for="custom_field_content" title="'.__( 'Post content', 'custom-post-types' ).'"><strong>'.__( 'Post content', 'custom-post-types' ).'</strong></label><br>
    <input type="text" class="shortcode-custom-fields" value=\'[custom-field id="content"]\' readonly="readonly" onfocus="this.select();" id="custom_field_content">
</p>';
                        echo '<p>
    <label for="custom_field_excerpt" title="'.__( 'Post excerpt', 'custom-post-types' ).'"><strong>'.__( 'Post excerpt', 'custom-post-types' ).'</strong></label><br>
    <input type="text" class="shortcode-custom-fields" value=\'[custom-field id="excerpt"]\' readonly="readonly" onfocus="this.select();" id="custom_field_excerpt">
</p>';
                        echo '<p>
    <label for="custom_field_image" title="'.__( 'Post image', 'custom-post-types' ).'"><strong>'.__( 'Post image', 'custom-post-types' ).'</strong></label><br>
    <input type="text" class="shortcode-custom-fields" value=\'[custom-field id="image"]\' readonly="readonly" onfocus="this.select();" id="custom_field_image">
</p>';
                        echo '<p>
    <label for="custom_field_date" title="'.__( 'Post date', 'custom-post-types' ).'"><strong>'.__( 'Post date', 'custom-post-types' ).'</strong></label><br>
    <input type="text" class="shortcode-custom-fields" value=\'[custom-field id="date"]\' readonly="readonly" onfocus="this.select();" id="custom_field_date">
</p>';
                        echo '<p>
    <label for="custom_field_author" title="'.__( 'Post author', 'custom-post-types' ).'"><strong>'.__( 'Post author', 'custom-post-types' ).'</strong></label><br>
    <input type="text" class="shortcode-custom-fields" value=\'[custom-field id="author"]\' readonly="readonly" onfocus="this.select();" id="custom_field_author">
</p>';
                        echo "<p style=\"text-align: center; text-transform: uppercase;\"><strong>".__( 'Custom field', 'custom-post-types' )."</strong></p>";


                        foreach($fields as $id => $field){ if($field['type'] !== '0'){

?><p>
    <label for="custom_field_<?php echo $id;?>" title="<?php echo $field['name'];?>"><strong><?php echo $field['name'];?></strong></label><br>
    <input type="text" class="shortcode-custom-fields" value='[custom-field id="<?php echo $id;?>"]' readonly="readonly" onfocus="this.select();" id="custom_field_<?php echo $id;?>">
</p>
<?php } } } } } ?>

<?php
        }



        public function render_custom_metabox( $post, $metabox ) {
            wp_nonce_field( 'custom_inner_metabox', 'custom_metabox_nonce' );

            $metabox_id = explode('_',$metabox['id'])[0];
            $custom_metabox = get_post($metabox_id);
            $all_fields = get_post_meta( $custom_metabox->ID, '_field_settings_meta')[0]['fields'] ? get_post_meta( $custom_metabox->ID, '_field_settings_meta')[0]['fields'] : array();

            foreach($all_fields as $key => $field){

                $type = isset($field['type']) ? $field['type'] : null;
                $label = isset($field['name']) ? $field['name'] : null;
                $id = $key;
                $required = isset($field['required']) ? ' required' : '';
                $options = isset($field['options']) ? $field['options'] : array();

                $value = isset(get_post_meta( $post->ID, '_custom_meta')[0][$id]) ? get_post_meta( $post->ID, '_custom_meta')[0][$id] : '';

                if($label){

?>

<div class="wp-field-row">
    <label for="<?php echo $id;?>_field" title="<?php echo $label; ?>">
        <?php echo $label; ?>
    </label>
    <?php if($type == 'text'){ ?>
    <input type="text" id="<?php echo $id;?>_field" name="custom_field[<?php echo $id;?>]" value="<?php echo $value;?>"<?php echo $required;?>/>
    <?php } elseif($type == 'number'){ ?>
    <input type="number" id="<?php echo $id;?>_field" name="custom_field[<?php echo $id;?>]" value="<?php echo $value;?>"<?php echo $required;?>/>
    <?php } elseif($type == 'email'){ ?>
    <input type="email" id="<?php echo $id;?>_field" name="custom_field[<?php echo $id;?>]" value="<?php echo $value;?>"<?php echo $required;?>/>
    <?php } elseif($type == 'textarea'){ ?>
    <textarea id="<?php echo $id;?>_field" name="custom_field[<?php echo $id;?>]"<?php echo $required;?>><?php echo $value;?></textarea>
    <?php } elseif($type == 'dropdown'){ ?>
    <select id="<?php echo $id;?>_field" name="custom_field[<?php echo $id;?>]"<?php echo $required;?>>
        <option value=""><?php _e( '- Choose -', 'custom-post-types' ); ?></option>
        <?php
                                        $options = explode(PHP_EOL, $options);    
                                        foreach($options as $option){ ?>
        <option value="<?php echo $option; ?>"<?php selected( $value, $option ); ?>><?php echo $option; ?></option>
        <?php } ?>
    </select>
    <?php } elseif($type == 'date'){ ?>
    <input type="date" id="<?php echo $id;?>_field" name="custom_field[<?php echo $id;?>]" value="<?php echo $value;?>"<?php echo $required;?>/>
    <?php } elseif($type == 'time'){ ?>
    <input type="time" id="<?php echo $id;?>_field" name="custom_field[<?php echo $id;?>]" value="<?php echo $value;?>"<?php echo $required;?>/>
    <?php } elseif($type == 'file'){ ?>
    <div class="file-uploader-field">
        <input id="<?php echo $id;?>_field" type="text" name="custom_field[<?php echo $id;?>]" value="<?php echo $value;?>"<?php echo $required;?>/>
        <input type="button" class="button-primary" value="<?php _e( 'Select', 'custom-post-types' ); ?>" data-file="<?php echo $id;?>_field" />
    </div>

    <?php } ?>
</div>

<?php }} ?>

<?php }




        public function init_cpt_ui_save_metabox( $post_id ) {
            $post = get_post($post_id);
            $post_type = $post->post_type;
            if ( $post_type == $this->cpt_ui_name ) {
                if ( ! isset( $_POST['cpt_ui_metabox_nonce'] ) ) {
                    return $post_id;
                }
                $nonce = $_POST['cpt_ui_metabox_nonce'];
                if ( ! wp_verify_nonce( $nonce, 'cpt_ui_inner_metabox' ) ) {
                    return $post_id;
                }
                if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
                    return $post_id;
                }
                if ( ! current_user_can( 'activate_plugins', $post_id ) ) {
                    return $post_id;
                }
                $mydata = $_POST['cpt_settings'];
                update_post_meta( $post_id, '_cpt_settings_meta', $mydata );

                flush_rewrite_rules();
            } elseif ( $post_type == $this->cpt_ui_name . '_tax' ) {
                if ( ! isset( $_POST['tax_ui_metabox_nonce'] ) ) {
                    return $post_id;
                }
                $nonce = $_POST['tax_ui_metabox_nonce'];
                if ( ! wp_verify_nonce( $nonce, 'tax_ui_inner_metabox' ) ) {
                    return $post_id;
                }
                if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
                    return $post_id;
                }
                if ( ! current_user_can( 'activate_plugins', $post_id ) ) {
                    return $post_id;
                }
                $mydata = $_POST['tax_settings'];
                update_post_meta( $post_id, '_tax_settings_meta', $mydata );

                flush_rewrite_rules();
            } elseif ( $post_type == $this->cpt_ui_name . '_field' ) {
                if ( ! isset( $_POST['field_ui_metabox_nonce'] ) ) {
                    return $post_id;
                }
                $nonce = $_POST['field_ui_metabox_nonce'];
                if ( ! wp_verify_nonce( $nonce, 'field_ui_inner_metabox' ) ) {
                    return $post_id;
                }
                if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
                    return $post_id;
                }
                if ( ! current_user_can( 'activate_plugins', $post_id ) ) {
                    return $post_id;
                }
                $mydata = $_POST['field_settings'];
                update_post_meta( $post_id, '_field_settings_meta', $mydata );

                flush_rewrite_rules();
            } elseif ( $post_type == $this->cpt_ui_name . '_template' ) {
                if ( ! isset( $_POST['template_ui_metabox_nonce'] ) ) {
                    return $post_id;
                }
                $nonce = $_POST['template_ui_metabox_nonce'];
                if ( ! wp_verify_nonce( $nonce, 'template_ui_inner_metabox' ) ) {
                    return $post_id;
                }
                if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
                    return $post_id;
                }
                if ( ! current_user_can( 'activate_plugins', $post_id ) ) {
                    return $post_id;
                }
                $mydata = $_POST['template_settings'];
                update_post_meta( $post_id, '_template_settings_meta', $mydata );

                flush_rewrite_rules();
            }

            if ( isset( $_POST['custom_metabox_nonce'] ) ) {
                $nonce = $_POST['custom_metabox_nonce'];
                if ( ! wp_verify_nonce( $nonce, 'custom_inner_metabox' ) ) {
                    return $post_id;
                }
                if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
                    return $post_id;
                }
                if ( ! current_user_can( 'activate_plugins', $post_id ) ) {
                    return $post_id;
                }
                $mydata = $_POST['custom_field'];
                update_post_meta( $post_id, '_custom_meta', $mydata );
            }
        }

        public function init_created_cpt(){

            $created_post_types = get_posts( array('posts_per_page' => -1, 'post_type' => $this->cpt_ui_name) );

            foreach($created_post_types as $post_type){

                $cpt_settings = get_post_meta( $post_type->ID, '_cpt_settings_meta')[0];

                $labels = array(
                    'name'               => __( $post_type->post_title, 'custom-post-types' ),
                    'singular_name'      => __( $cpt_settings['cpt_singular_name'], 'custom-post-types' ),
                    'menu_name'          => __( $post_type->post_title, 'custom-post-types' ),
                    'name_admin_bar'     => __( $cpt_settings['cpt_singular_name'], 'custom-post-types' ),
                    'add_new'            => sprintf( __( 'Add new %s', 'custom-post-types' ), $cpt_settings['cpt_singular_name'] ),
                    'add_new_item'       => sprintf( __( 'Add new %s', 'custom-post-types' ), $cpt_settings['cpt_singular_name'] ),
                    'new_item'           => sprintf( __( 'Add %s', 'custom-post-types' ), $cpt_settings['cpt_singular_name'] ),
                    'edit_item'          => sprintf( __( 'Edit %s', 'custom-post-types' ), $cpt_settings['cpt_singular_name'] ),
                    'view_item'          => sprintf( __( 'View %s', 'custom-post-types' ), $cpt_settings['cpt_singular_name'] ),
                    'all_items'          => sprintf( __( 'View %s', 'custom-post-types' ), $post_type->post_title ),
                    'search_items'       => sprintf( __( 'Search %s', 'custom-post-types' ), $post_type->post_title ),
                    'not_found'          => __( 'No Contents avaiable.', 'custom-post-types' ),
                    'not_found_in_trash' => __( 'No Contents in the trash.', 'custom-post-types' )
                );

                $supports = array('title');

                if( isset($cpt_settings['cpt_editor']) && $cpt_settings['cpt_editor'] == 1 ){
                    array_push($supports, 'editor');
                }
                if( isset($cpt_settings['cpt_excerpt']) && $cpt_settings['cpt_excerpt'] == 1 ){
                    array_push($supports, 'excerpt');
                }
                if( isset($cpt_settings['cpt_thumbnail']) && $cpt_settings['cpt_thumbnail'] == 1 ){
                    array_push($supports, 'thumbnail');
                }

                $args = array(
                    'labels'             => $labels,
                    'description'        => __( 'Post type created with the "Custom post type" plugin.', 'custom-post-types' ),
                    'public'             => $cpt_settings['cpt_public'] == 0 ? false : true,
                    'publicly_queryable' => $cpt_settings['cpt_public'] == 0 ? false : true,
                    'show_ui'            => true,
                    'show_in_menu'       => true,
                    'query_var'          => $cpt_settings['cpt_public'] == 0 ? false : true,
                    'rewrite'            => $cpt_settings['cpt_public'] == 0 ? false : array( 'slug' => sanitize_title($post_type->post_title) ),
                    'capabilities' => array(
                        'edit_post'          => $cpt_settings['cpt_role'] == 0 ? 'edit_posts' : 'update_core',
                        'read_post'          => $cpt_settings['cpt_role'] == 0 ? 'edit_posts' : 'update_core',
                        'delete_post'        => $cpt_settings['cpt_role'] == 0 ? 'edit_posts' : 'update_core',
                        'edit_posts'         => $cpt_settings['cpt_role'] == 0 ? 'edit_posts' : 'update_core',
                        'edit_others_posts'  => $cpt_settings['cpt_role'] == 0 ? 'edit_posts' : 'update_core',
                        'delete_posts'       => $cpt_settings['cpt_role'] == 0 ? 'edit_posts' : 'update_core',
                        'publish_posts'      => $cpt_settings['cpt_role'] == 0 ? 'edit_posts' : 'update_core',
                        'read_private_posts' => $cpt_settings['cpt_role'] == 0 ? 'edit_posts' : 'update_core'
                    ),
                    'has_archive'        => $cpt_settings['cpt_public'] == 0 ? false : true,
                    'hierarchical'       => $cpt_settings['cpt_hierarchical'] == 0 ? false : true,
                    'menu_position'      => null,
                    'supports'           => $supports,
                    'menu_icon'          => $cpt_settings['cpt_icon'] ? $cpt_settings['cpt_icon'] : 'dashicons-tag'
                );
                register_post_type( 'cpt_' . $post_type->ID , $args );

            }

        }

        public function init_created_tax(){

            $created_taxonomies = get_posts( array('posts_per_page' => -1, 'post_type' => $this->cpt_ui_name . '_tax') );

            foreach($created_taxonomies as $taxonomy){

                $tax_settings = get_post_meta( $taxonomy->ID, '_tax_settings_meta')[0];

                $labels = array(
                    'name'              => __( $taxonomy->post_title, 'custom-post-types' ),
                    'singular_name'     => __( $tax_settings['tax_singular_name'], 'custom-post-types' ),
                    'search_items'      => sprintf( __( 'Search %s', 'custom-post-types' ), $taxonomy->post_title ),
                    'all_items'         => sprintf( __( 'View %s', 'custom-post-types' ), $taxonomy->post_title ),
                    'parent_item'       => sprintf( __( 'Parent %s', 'custom-post-types' ), $tax_settings['tax_singular_name'] ),
                    'parent_item_colon' => sprintf( __( 'Parent %s:', 'custom-post-types' ), $tax_settings['tax_singular_name'] ),
                    'edit_item'         => sprintf( __( 'Edit %s', 'custom-post-types' ), $tax_settings['tax_singular_name'] ),
                    'update_item'       => sprintf( __( 'Update %s', 'custom-post-types' ), $tax_settings['tax_singular_name'] ),
                    'add_new_item'      => sprintf( __( 'Add %s', 'custom-post-types' ), $tax_settings['tax_singular_name'] ),
                    'new_item_name'     => sprintf( __( '%s name', 'custom-post-types' ), $tax_settings['tax_singular_name'] ),
                    'menu_name'         => __( $taxonomy->post_title, 'custom-post-types' ),
                );

                $args = array(
                    'hierarchical'      => $tax_settings['tax_hierarchical'] == 0 ? false : true,
                    'labels'            => $labels,
                    'show_ui'           => true,
                    'show_admin_column' => true,
                    'public'            => $tax_settings['tax_public'] == 0 ? false : true,
                    'query_var'         => $tax_settings['tax_public'] == 0 ? false : true,
                    'rewrite'           => $tax_settings['tax_public'] == 0 ? false : array( 'slug' => sanitize_title($taxonomy->post_title) ),
                    'capabilities' => array(
                        'manage_terms' => $tax_settings['tax_role'] == 0 ? 'edit_posts' : 'update_core',
                        'edit_terms' => $tax_settings['tax_role'] == 0 ? 'edit_posts' : 'update_core',
                        'delete_terms' => $tax_settings['tax_role'] == 0 ? 'edit_posts' : 'update_core',
                        'assign_terms' => $tax_settings['tax_role'] == 0 ? 'edit_posts' : 'update_core',
                    )
                );

                register_taxonomy( 'tax_' . $taxonomy->ID , $tax_settings['tax_post_types'] ? $tax_settings['tax_post_types'] : array(), $args );

            }

        }

        public function init_created_template($load_template){

            $current_theme_name = wp_get_theme()->get( 'TextDomain' );
            $upload = wp_upload_dir();
            $upload_dir = $upload['basedir'];
            $current_teme_single_template = $upload_dir . '/custom-templates/' . $current_theme_name .'-single.php';

            if (is_single()){
                global $post;
                $created_templates = get_posts( array('posts_per_page' => -1, 'post_type' => $this->cpt_ui_name . '_template') );
                foreach($created_templates as $template){
                    $template_settings = get_post_meta( $template->ID, '_template_settings_meta')[0];
                    $template_used_by = isset($template_settings['field_used_by']) ? $template_settings['field_used_by'] : '';
                    if($template_used_by === $post->post_type && file_exists($current_teme_single_template)){
                        $load_template = $current_teme_single_template;
                    }                    
                }
            }

            return $load_template;
        }

        public function init_admin_form_top() {
            $screen = get_current_screen();
            if( ($screen->post_type == $this->cpt_ui_name && $screen->id == $this->cpt_ui_name) || ($screen->post_type == $this->cpt_ui_name . '_tax' && $screen->id == $this->cpt_ui_name . '_tax')  ) { ?>
<div class="<?php echo $this->cpt_ui_name;?>_top">
    <label for="title"><?php _e( 'Name', 'custom-post-types' ); ?></label>
</div>
<?php }
        }

        public function init_admin_form_sub() {
            $screen = get_current_screen();
            if( ($screen->post_type == $this->cpt_ui_name && $screen->id == $this->cpt_ui_name) || ($screen->post_type == $this->cpt_ui_name . '_tax' && $screen->id == $this->cpt_ui_name . '_tax')  ) { ?>
<div class="<?php echo $this->cpt_ui_name;?>_sub">
    <small><?php _e( 'It must be unique. It will be used in the menu and as a slug in the URLs.', 'custom-post-types' ); ?></small>
</div>
<?php }
        }

        public function work_in_progress() {
?>
<div class="wrap">
    <h1><?php echo __( 'Work in progress...', 'custom-post-types' );?></h1>

    <p><?php echo __( 'I am working to implement this feature which will be free to all users. <br> Below is the donation link to help me support processing costs and the link to my website.', 'custom-post-types' );?></p>
    <p><a href="https://www.andreadegiovine.it/outlinks/1422/?utm_source=wordpress_org&utm_medium=plugin_page&utm_campaign=<?php echo (!empty($_GET['page']) ? $_GET['page'] : 'installed_plugin' );?>" class="button button-primary" target="_blank"><?php echo __( 'Send donation', 'custom-post-types' );?></a></p>
    <p><a href="https://www.andreadegiovine.it/" class="button button-primary" target="_blank"><?php echo __( 'Developer website', 'custom-post-types' );?></a></p>
    <p><a href="https://www.andreadegiovine.it/invia-suggerimento?oggetto=Custom%20Post%20Types%20Plugin" class="button button-primary" target="_blank"><?php echo __( 'Send suggestion', 'custom-post-types' );?></a></p>


</div>
<?php }

        public function init_cpt_ui_unique_name( $data , $postarr ){
            if( count(get_posts( array('posts_per_page' => 2, 'post_type'  => $data['post_type'], 'title' => $data['post_title'], 'exclude' => $postarr['ID']) )) >= 1 && ($data['post_type'] == $this->cpt_ui_name . '_tax' || $data['post_type'] == $this->cpt_ui_name )){
                $data['post_title'] .= ' ' . $postarr['ID'];
            }
            return $data;
        }

        public function init_cpt_admin_columns($columns){
            global $pagenow;
            if( 'edit.php' === $pagenow && isset($_GET['post_type']) && $_GET['post_type'] === $this->cpt_ui_name ){
                $date = $columns['date'];
                unset($columns['date']);
                $columns['cpt_id'] = __( 'ID', 'custom-post-types' );
                $columns['cpt_base'] = __( 'Permalink base', 'custom-post-types' );
                $columns['cpt_count'] = __( 'Count', 'custom-post-types' );
                $columns['date'] = $date;
            } elseif( 'edit.php' === $pagenow && isset($_GET['post_type']) && $_GET['post_type'] === $this->cpt_ui_name . '_tax' ){
                $date = $columns['date'];
                unset($columns['date']);
                $columns['cpt_id'] = __( 'ID', 'custom-post-types' );
                $columns['cpt_base'] = __( 'Permalink base', 'custom-post-types' );
                $columns['cpt_used'] = __( 'Used by', 'custom-post-types' );
                $columns['date'] = $date;
            } elseif( 'edit.php' === $pagenow && isset($_GET['post_type']) && $_GET['post_type'] === $this->cpt_ui_name . '_field' ){
                $date = $columns['date'];
                unset($columns['date']);
                $columns['cpt_used'] = __( 'Used by', 'custom-post-types' );
                $columns['date'] = $date;
            }

            return $columns;
        }

        public function init_render_cpt_admin_columns( $column, $post_id ) {
            global $pagenow;

            if( 'edit.php' === $pagenow && isset($_GET['post_type']) && $_GET['post_type'] === $this->cpt_ui_name ){
                switch ( $column ) {

                    case 'cpt_id' :
                        echo 'cpt_' . $post_id;
                        break;


                    case 'cpt_base' :
                        $cpt_settings = get_post_meta( $post_id, '_cpt_settings_meta')[0];
                        echo ( $cpt_settings['cpt_public'] == 0 ? '-' : '/' . sanitize_title(get_post($post_id)->post_title) . '/' );
                        break;

                    case 'cpt_count' :
                        echo '<a href="' . admin_url( 'edit.php?post_type=cpt_' . $post_id ) . '">' . wp_count_posts('cpt_' . $post_id)->publish . '</a>'; 
                        break;

                }
            } elseif( 'edit.php' === $pagenow && isset($_GET['post_type']) && $_GET['post_type'] === $this->cpt_ui_name . '_tax' ){
                switch ( $column ) {

                    case 'cpt_id' :
                        echo 'tax_' . $post_id;
                        break;


                    case 'cpt_base' :
                        $tax_settings = get_post_meta( $post_id, '_tax_settings_meta')[0];
                        echo ( $tax_settings['tax_public'] == 0 ? '-' : '/' . sanitize_title(get_post($post_id)->post_title) . '/' );
                        break;

                    case 'cpt_used' :
                        global $wp_taxonomies;
                        if(isset( $wp_taxonomies['tax_' . $post_id] )){
                            $output = array();
                            foreach($wp_taxonomies['tax_' . $post_id]->object_type as $post_type){
                                $obj = get_post_type_object( $post_type );
                                if($obj){
                                    $output[] = '<a href="' . admin_url( 'edit.php?post_type=' . $post_type ) . '">' . $obj->labels->name . '</a>';
                                }
                            }
                            echo (!empty($output) ? implode(', ',$output) : '-');
                        } else {
                            echo '-';
                        }
                        break;

                }
            } elseif( 'edit.php' === $pagenow && isset($_GET['post_type']) && $_GET['post_type'] === $this->cpt_ui_name . '_field' ){
                switch ( $column ) {

                    case 'cpt_used' :
                        $field_settings = isset(get_post_meta( $post_id, '_field_settings_meta')[0]) ? get_post_meta( $post_id, '_field_settings_meta')[0] : array();
                        $field_post_types = isset($field_settings['field_post_types']) ? $field_settings['field_post_types'] : array();


                        if(!empty($field_post_types)){
                            $output = array();
                            foreach($field_post_types as $post_type){
                                $obj = get_post_type_object( $post_type );
                                if($obj){
                                    $output[] = '<a href="' . admin_url( 'edit.php?post_type=' . $post_type ) . '">' . $obj->labels->name . '</a>';
                                }
                            }
                            echo (!empty($output) ? implode(', ',$output) : '-');
                        } else {
                            echo '-';
                        }


                        //print_r($field_post_types);

                        //_e( 'Work in progress...', 'custom-post-types' );
                        break;

                }
            }
        }

        public function init_admin_notices(){
            if(!isset($_COOKIE[ $this->cpt_ui_name . '_notices_close'])) { ?>
<?php printf( '<div class="%1$s"><p>%2$s<br><a href="%3$s" target="_blank">%4$s</a> - <a href="%5$s" target="_blank">%6$s</a> - <a href="#" class="%7$s-notices-close">%8$s</a></p></div>', $this->cpt_ui_name . '-notices notice notice-success is-dismissible', __( '<strong>Custom post type</strong> notice:<br>Thanks for using this plugin! Do you want to help us grow to add new features?', 'custom-post-types' ) , 'https://wordpress.org/support/plugin/custom-post-types/reviews/#new-post', __( 'Review this plugin', 'custom-post-types' ), 'https://www.andreadegiovine.it/outlinks/1422/?utm_source=wordpress_dashboard&utm_medium=notices&utm_campaign=custom_post_types', __( 'Send donation', 'custom-post-types' ), $this->cpt_ui_name, __( 'Close and don\'t show again', 'custom-post-types' ) ); ?>
<script>
    function createCookie(name, value, days) {
        var expires;
        if (days) {
            var date = new Date();
            date.setTime(date.getTime() + (days * 24 * 60 * 60 * 1000));
            expires = "; expires=" + date.toGMTString();
        } else {
            expires = "";
        }
        document.cookie = encodeURIComponent(name) + "=" + encodeURIComponent(value) + expires + "; path=/";
    }
    jQuery(document).ready(function ($) {
        $( ".<?php echo $this->cpt_ui_name;?>-notices .<?php echo $this->cpt_ui_name;?>-notices-close" ).click(function(e) {
            e.preventDefault();
            createCookie('<?php echo $this->cpt_ui_name;?>_notices_close',1,7);
            $( ".<?php echo $this->cpt_ui_name;?>-notices .notice-dismiss" ).click();
        });
    });
</script>
<?php }

            $current_theme_name = wp_get_theme()->get( 'TextDomain' );
            $upload = wp_upload_dir();
            $upload_dir = $upload['basedir'];
            $current_teme_single_template = $upload_dir . '/custom-templates/' . $current_theme_name .'-single.php';
            $theme_single_template = file_get_contents(get_template_directory() . '/single.php');
            if(!file_exists(get_template_directory() . '/single.php')  && get_post_type() === $this->cpt_ui_name . '_template') {
                printf( '<div class="%1$s"><p>%2$s<br><a href="%3$s" class="button-primary" style="margin-top:1em;" target="_blank">%4$s</a></p></div>', $this->cpt_ui_name . '-notices notice notice-error', __( '<strong>Custom post type</strong> notice:<br>The theme you are using does not contain the "single.php" file in the main theme folder. To use custom templates the theme must have the "single.php" file.', 'custom-post-types' ) , 'https://developer.wordpress.org/themes/template-files-section/post-template-files/', __( 'WordPress documentation', 'custom-post-types' ) );
            } elseif(!preg_match('#while(.+)endwhile#s',$theme_single_template,$matches) && get_post_type() === $this->cpt_ui_name . '_template'){
                printf( '<div class="%1$s"><p>%2$s<br><a href="%3$s" class="button-primary" style="margin-top:1em;" target="_blank">%4$s</a></p></div>', $this->cpt_ui_name . '-notices notice notice-error', __( '<strong>Custom post type</strong> notice:<br>The theme you are using does not contain the "while () : ... endwhile;" loop in the "single.php" file. To use custom templates the theme must have this "while" loop type in its "single.php" file.', 'custom-post-types' ) , 'https://codex.wordpress.org/The_Loop#Loop_Examples', __( 'WordPress documentation', 'custom-post-types' ) );
            } elseif (!file_exists($current_teme_single_template) && get_post_type() === $this->cpt_ui_name . '_template') {
                printf( '<div class="%1$s"><p>%2$s<br><a href="%3$s" class="button-primary" style="margin-top:1em;">%4$s</a></p></div>', $this->cpt_ui_name . '-notices notice notice-error', __( '<strong>Custom post type</strong> notice:<br>Is this the first time you use the plugin or have you changed the theme?<br>In order to use custom templates, the plugin must copy the "single.php" file of the current theme to the "custom-templates" folder contained in the WordPress "uploads" folder.', 'custom-post-types' ) , admin_url( 'edit.php?post_type='.$this->cpt_ui_name.'_template&action=copy_theme_template' ), __( 'Continue', 'custom-post-types' ) );
            }
        }

        public function init_admin_action_links($links, $file){
            if ( $file == 'custom-post-types/custom-post-types.php' ) {
                $links[] = sprintf( '<a href="%s" target="_blank"> %s </a>', 'https://wordpress.org/support/plugin/custom-post-types', __( 'Support', 'custom-post-types' ) );
                $links[] = sprintf( '<a href="%s" target="_blank"> %s </a>', 'https://wordpress.org/support/plugin/custom-post-types/reviews/#new-post', __( 'Review', 'custom-post-types' ) );
            }
            return $links;
        }

        public function field_shortcode($atts){
            $a = shortcode_atts( array(
                'id' => null,
            ), $atts );

            if(!$a['id']){
                return;
            }

            global $post;

            if($a['id'] == 'title'){
                return $post->post_title;
            } elseif($a['id'] == 'content'){
                return $post->post_content;
            } elseif($a['id'] == 'excerpt'){
                return $post->post_excerpt;
            } elseif($a['id'] == 'image'){
                return get_the_post_thumbnail_url($post->ID,'full');
            } elseif($a['id'] == 'date'){
                return get_the_date( get_option( 'date_format' ), $post->ID );
            } elseif($a['id'] == 'author'){
                $author_id=$post->post_author;
                return get_the_author_meta( 'user_nicename' , $author_id );
            }

            $value = isset(get_post_meta( $post->ID, '_custom_meta')[0][$a['id']]) ? get_post_meta( $post->ID, '_custom_meta')[0][$a['id']] : '';

            return $value;

        }

    }

    new adg_custom_post_types();
}

function init_plugin_dev_functions(){

    if (!function_exists('get_custom_field')) {
        function get_custom_field($field_id = null, $post_id = null){
            global $post;
            if(!$field_id || is_admin() || !is_single()){
                return;
            }
            if(!$post_id || !get_post($post_id)){
                $post_id = $post->ID;
            }            
            $value = isset(get_post_meta( $post_id, '_custom_meta')[0][$field_id]) ? get_post_meta( $post_id, '_custom_meta')[0][$field_id] : '';
            return $value;
        }
    }

    if (!function_exists('custom_post_types_get_custom_template')) {
        function custom_post_types_get_custom_template(){
            if (is_single()){
                global $post;
                $created_templates = get_posts( array('posts_per_page' => -1, 'post_type' => 'manage_cpt_template') );
                foreach($created_templates as $template){
                    $template_settings = get_post_meta( $template->ID, '_template_settings_meta')[0];
                    $template_used_by = isset($template_settings['field_used_by']) ? $template_settings['field_used_by'] : '';
                    if($template_used_by === $post->post_type){
                        echo do_shortcode($template->post_content);
                    }                    
                }
            }
        }
    }

}
add_action('init', 'init_plugin_dev_functions');

